#include "p30f6014A.h"

#define IR_RECIEVER

#include "stdio.h"
#include "string.h"
#include "math.h"
#include <time.h>
#include <stdlib.h>

#include <codec/e_sound.h>
#include <motor_led/e_init_port.h>
#include <motor_led/advance_one_timer/e_led.h>
#include <motor_led/advance_one_timer/e_motors.h>
#include <motor_led/advance_one_timer/e_agenda.h>
#include <camera/fast_2_timer/e_poxxxx.h>
#include <uart/e_uart_char.h>
#include <a_d/advance_ad_scan/e_ad_conv.h>
#include <a_d/advance_ad_scan/e_prox.h>
#include <a_d/advance_ad_scan/e_acc.h>
#include <a_d/advance_ad_scan/e_micro.h>
#include <bluetooth/e_bluetooth.h>
#include <motor_led/e_epuck_ports.h>
#include <I2C/e_I2C_protocol.h>
#include <camera/fast_2_timer/e_po6030k.h>
#include "memory.h"

/* selector on normal extension*/

#define SELECTOR0 _RG6

#define SELECTOR1 _RG7

#define SELECTOR2 _RG8

#define SELECTOR3 _RG9

#ifdef IR_RECIEVER
#include <motor_led/advance_one_timer/e_remote_control.h>
#define SPEED_IR 600
#endif 
#include "runaccelerometer.h"
#include "runbreitenberg_adv.h"
#include "runlocatesound.h"
#include "runwallfollow.h"
#include "rungrounddirection.h"
#include "utility.h"
#include "Asercom.h"
#include "runshow.h"
#include "translatorI2C.h"

#define PI 3.14159265358979

#define uart2_send_static_text(msg) { e_send_uart2_char(msg,sizeof(msg)-1); while(e_uart2_sending()); }
#define uart1_send_static_text(msg) { e_send_uart1_char(msg,sizeof(msg)-1); while(e_uart1_sending()); }

#define PROXSCALING_SHOCK 2 //4
//#define BASICSPEED 550
unsigned int basicSpeed = 550;

#define GROUND_THR 550

#define LEFT_ROT  1 
#define RIGHT_ROT 2
#define CLIFF_RIGHT 0
#define FLOOR_RIGHT 1
#define FLOOR_CENTER 2
#define FLOOR_LEFT 3
#define CLIFF_LEFT 4
#define FALL_THR 400

int s, m;
long potential[2];
int speed[2];
int matrix_prox[2][8] =
	{{8,4,2,0,0,-4,-8,-16},
	{-16,-8,-4,0,0,2,4,8}};
int timerCount=0;
unsigned int groundValues[5] = {1000};
char groundTemp[10] = {0};
unsigned char uartBuff[50]={0};
unsigned char demoState = 0;
unsigned char lineFound = 0;
unsigned int outOfLine = 0;
unsigned char chargeContact = 0;
int camMode=GREY_SCALE_MODE,camWidth=1,camHeight=1,camZoom=1;
unsigned char camImage[1];
unsigned char randomSound = 0;
unsigned char randomLed = 0;
unsigned char randomSpeed = 0;
unsigned char randomDirection = 0;
unsigned char randomRotation = 0;
unsigned int lastTimerCount = 0;
unsigned int currentSelector = 0;
unsigned char directionChanged=0;
signed int lineFollowSpeed=0;
//unsigned char regValue = 0;
//char buffer[BUFFER_SIZE];
//unsigned char ledDetected = 0;
//unsigned int pixelNum = 0;
//unsigned char maxPixel = 0;
unsigned int k=0, l=0;
unsigned int floor_right=0, floor_center=0, floor_left=0, cliff_left=1024, cliff_right=1024, min_sensor_value=1024;
int delta=0, speedr=0, speedl=0, selector=0;
char prev_rot = 0, min_sensor=CLIFF_RIGHT;
int jGround=0, iGround=0;
unsigned char escapeDir = 0;
unsigned char timerTen = 0;

/*! \brief Calcul the speed to set on each wheel for avoiding
 *
 * Here we do a level-headed sum to take advantage of each captor
 * depending of there position. For exemple if the captor number 0
 * detect something, he has to set the right speed high and set 
 * the left speed low.
 */
void shock_neuron(void)
{

	if(demoState>=2) {	// disable obstacle avoidance during line following otherwise we cannot reach the charger...and also during charge
		return;
	} else {

		for (m = 0; m < 2; m++)
		{
			potential[m] = 0;
			for (s = 0; s < 8; s++)
				potential[m] += (matrix_prox[m][s]*e_get_calibrated_prox(s)); // get values from proximity sensors
	        speed[m] = (potential[m]/PROXSCALING_SHOCK + basicSpeed);
		}
	
		if((speed[1] < 50 && speed[1] > -50)
			&& (speed[0] < 50 && speed[0] > -50)) {
			speed[1] = speed[1] * 20;
			speed[0] = speed[0] * 20;
		}
	
		if (speed[1] > 1000)
			speed[1] = 1000;
		else if (speed[1] < -1000 )
			speed[1] = -1000;
	
		if (speed[0] > 1000)
			speed[0] = 1000;
		else if (speed[0] < -1000 )
			speed[0] = -1000;
	
		e_set_speed_left(speed[1]);
		e_set_speed_right(speed[0]);

	}
}

void myTimer() {
	timerCount++;
}

void updateGround() {

	jGround=0; 
	iGround=0;

	e_i2cp_enable();
	for(jGround = 0; jGround < 6; jGround++) {
		if (jGround % 2 == 0) { 
			groundTemp[iGround++] = e_i2cp_read(0xC0, jGround + 1);
		} else {
			groundTemp[iGround++] = e_i2cp_read(0xC0, jGround - 1);
		}
	}
	for(jGround=13; jGround<17; jGround++) {
    	if (jGround % 2 == 0) {
			groundTemp[jGround-7] = e_i2cp_read(0xC0, jGround - 1);
		} else {
			groundTemp[jGround-7] = e_i2cp_read(0xC0, jGround + 1);	          				
		}
	}    
	e_i2cp_disable();

	groundValues[0] = (unsigned int) (groundTemp[0] & 0xff) + ((unsigned int) groundTemp[1] << 8);
	groundValues[1] = (unsigned int) (groundTemp[2] & 0xff) + ((unsigned int) groundTemp[3] << 8);
	groundValues[2] = (unsigned int) (groundTemp[4] & 0xff) + ((unsigned int) groundTemp[5] << 8);
	groundValues[3] = (unsigned int) (groundTemp[6] & 0xff) + ((unsigned int) groundTemp[7] << 8);
	groundValues[4] = (unsigned int) (groundTemp[8] & 0xff) + ((unsigned int) groundTemp[9] << 8);
	//sprintf(uartBuff, "%d, %d, %d, %d, %d\n\r", groundValues[0], groundValues[1], groundValues[2], camImage[0], outOfLine);
	//uart1_send_static_text(uartBuff);

}

void resetVars() {
	if(demoState != 3) {	// keep the value of the timer when in state3 (charge) in order to avoid to pass from state2 (follow line)
							// to state3 (charge) and back to state2 without any timeout
		timerCount = 0;
	}
	lastTimerCount = 0;
	lineFound = 0;
	outOfLine = 0;
	chargeContact = 0;
	directionChanged = 0;
	lineFollowSpeed = 0;
	timerTen = 0;
}

char disable_transport_layer(void)
{
	char send[7];
	char read[30];
	int i;
	char c;
	
	send[0]=0x02; //send reset request
	send[1]=0x52;
	send[2]=0x52; //0x26;
	send[3]=0x00;
	send[4]=0x00;
	send[5]=0xA4;
	send[6]=0x03;
	e_send_uart1_char(send,7);
	
	i=0;
	c=0;
	
	 do
	    {
	      if (e_getchar_uart1(&read[i]))		//read response
			{	
				c=read[i];
	     		i++;
			}	
	    }
	    while (((char)c != 0x03)||(i<(read[3]+6)));
	    read[i]='\0';
	return read[6];
}

int main() {

	//system initialization 
	e_init_port();    // configure port pins
	e_start_agendas_processing();
	e_init_motors();
	e_init_uart1();   // initialize UART to 115200 Kbaud
	e_init_uart2();   // initialize UART to 115200 Kbaud
	e_init_ad_scan(ALL_ADC);
	e_i2cp_init();
	e_poxxxx_init_cam();
	e_poxxxx_config_cam((ARRAY_WIDTH-camWidth*camZoom)/2,(ARRAY_HEIGHT-camHeight*camZoom)/2,camWidth*camZoom,camHeight*camZoom,camZoom,camZoom,camMode);
	e_poxxxx_set_mirror(1,1);
	e_poxxxx_write_cam_registers();
	//regValue = e_po6030k_read_register(BANK_C, 0x04);
	//regValue |= (1 << 2);		// disable auto white balance
	//e_po6030k_write_register(BANK_C, 0x04, regValue);

	//e_init_sound();
	srand(TMR2);

	//Reset if Power on (some problem for few robots)
	if (RCONbits.POR) {
		RCONbits.POR=0;
		__asm__ volatile ("reset");
	}

	e_set_led(8, 1);
	//e_play_sound(11028,8016);
	//e_play_sound(7294,3732);
	e_calibrate_ir();
	e_led_clear();

	resetVars();
	demoState = 0;
	e_activate_agenda(updateGround, 200);	// 20 ms (50 Hz)
	e_activate_agenda(myTimer, 10000);	// every second
	
	while(1) {

		currentSelector = getselector();

		if(currentSelector==0) {

			switch(demoState) {
				case 0:	// move around
						e_set_body_led(0);
						e_set_front_led(0);

						if(timerCount >= 15) {	// about 15 seconds
							resetVars();
							demoState = 1;
							break;
						}	
	
						if((timerCount%5==0) && (lastTimerCount != timerCount)) {	// change state every 5 seconds
	
							lastTimerCount = timerCount;

							randomSpeed = rand()%9;
							switch(randomSpeed) {
								case 0: basicSpeed = 550;
										break;
								case 1: basicSpeed = 700;
										break;
								case 2: basicSpeed = 850;
										break;
								case 3: basicSpeed = 1000;
										break;
								case 4: basicSpeed = 1150;
										break;
								case 5: basicSpeed = 1250;
										break;
								case 6: basicSpeed = 1350;
										break;
								case 7: basicSpeed = 1450;
										break;
								case 8: basicSpeed = 1500;
										break;
							}

//							randomSound = rand()%5;
//							if(currentSelector==1) {
//								switch(randomSound) {
//									case 0: e_play_sound(0,2112);
//											break;
//									case 1: e_play_sound(2116,1760);
//											break;
//									case 2: e_play_sound(3878,3412);
//											break;
//									case 3: e_play_sound(7294,3732);
//											break;
//									case 4: e_play_sound(11028,8016);
//											break;
//								}
//							}

							randomRotation = rand()%3;
							if(randomRotation==1) {
								e_destroy_agenda(shock_neuron);
								e_set_steps_left(0);
								e_set_speed_left(-700);
								e_set_speed_right(700);
								while(e_get_steps_left() > -500);
								e_activate_agenda(shock_neuron, 650);							
							} else if(randomRotation==2) {
								e_destroy_agenda(shock_neuron);
								e_set_steps_right(0);
								e_set_speed_right(-700);
								e_set_speed_left(700);
								while(e_get_steps_right() > -500);
								e_activate_agenda(shock_neuron, 650);
							}

							switch(randomLed) {
								case 0: e_destroy_agenda(snake_led);
										break;
								case 1: e_destroy_agenda(flow_led);
										break;
								case 2: e_destroy_agenda(k2000_led);
										break;
								case 3: e_destroy_agenda(right_led);
										break;
								case 4: e_destroy_agenda(left_led);
										break;						
							}
							randomLed = rand()%3;
							switch(randomLed) {
								case 0: e_activate_agenda(snake_led, 1000);
										break;
								case 1: e_activate_agenda(flow_led, 1000);
										break;
								case 2: e_activate_agenda(k2000_led, 1000);
										break;
								case 3: e_activate_agenda(right_led, 1000);
										break;
								case 4: e_activate_agenda(left_led, 1000);
										break;						
							}
							

	
						}
						
						break;
	
				case 1:	// search for a line
						e_set_body_led(0);
						if(timerCount&0x01) {
							e_set_front_led(1);
						} else {
							e_set_front_led(0);
						}
						
						if(timerCount > 20) {	// the robot seems to be blocked somehow, so after about 20 seconds (with camera activated) restart from state=0
							resetVars();
							demoState = 0;
							break;
						}					
	
						if(outOfLine > 0) {		// go out of the line
							outOfLine--;
						} else {
							if(groundValues[0]<GROUND_THR || groundValues[1]<GROUND_THR || groundValues[2]<GROUND_THR || groundValues[3]<GROUND_THR || groundValues[4]<GROUND_THR) {
								lineFound++;
								if(lineFound > 10) {
									e_poxxxx_launch_capture(&camImage);	// start camera capture
									resetVars();
									demoState = 2;
									break;
								}
							} else {
								lineFound = 0;
							}
						}
	
						break;
	
				case 2:	// line found, follow it
						e_set_front_led(0);
						if(timerCount&0x01) {
							e_set_body_led(1);
						} else {
							e_set_body_led(0);
						}

						if(timerCount > 10) {	// the robot seems to be blocked somehow, try to escape...
							if(escapeDir == 0) {
								e_set_steps_left(0);
								e_set_speed_left(-150);
								e_set_speed_right(0);
								while(e_get_steps_left() > -75);
								e_set_steps_right(0);
								e_set_speed_left(0);
								e_set_speed_right(-150);
								while(e_get_steps_right() > -75);
							} else {
								e_set_steps_right(0);
								e_set_speed_left(0);
								e_set_speed_right(-150);
								while(e_get_steps_right() > -75);
								e_set_steps_left(0);
								e_set_speed_left(-150);
								e_set_speed_right(0);
								while(e_get_steps_left() > -75);
							}
							timerCount = 0;
							escapeDir = 1 - escapeDir;
							timerTen++;
						}

						if(timerTen >= 3) {	// the robot seems to be blocked somehow and we cannot escape, so after about 30 seconds (with camera activated) restart from state=0
							e_set_steps_left(0);
							e_set_speed_left(-300);
							e_set_speed_right(-300);
							while(e_get_steps_left() > -250);
							resetVars();
							demoState = 0;
							break;
						}
	
						while(!e_poxxxx_is_img_ready());	// wait end of capture

						if(camImage[0]>180) {
							chargeContact++;
							if(chargeContact > 10) {
								e_set_speed_left(0);
								e_set_speed_right(0);						
								e_poxxxx_launch_capture(&camImage);	// start camera capture
								resetVars();
								demoState = 3;
								break;
							}
						} else {	// the is turned off so the contact isn't good yet
							chargeContact = 0;
						}

						if(groundValues[0]>900 && groundValues[1]>900 && groundValues[2]>900) {	// i'm going to go out of the line
							outOfLine++;
							if(outOfLine > 10) {
								resetVars();
								outOfLine = 10000;
								demoState = 1;
								break;
							}
						} else {
							outOfLine = 0;
						}

						e_poxxxx_launch_capture(&camImage);	// start camera capture
	
						if(groundValues[4] < GROUND_THR && groundValues[0]>GROUND_THR && groundValues[1]>GROUND_THR && groundValues[2]>GROUND_THR && groundValues[3]>GROUND_THR) { // left cliff inside line, turn left
							e_set_speed_left(-300);
							e_set_speed_right(300);
							directionChanged=1;
						} else if(groundValues[3] < GROUND_THR && groundValues[0]>GROUND_THR && groundValues[1]>GROUND_THR && groundValues[2]>GROUND_THR && groundValues[4]>GROUND_THR) {
							e_set_speed_left(300);
							e_set_speed_right(-300);
							directionChanged=1;
						} else if(groundValues[0] > GROUND_THR && groundValues[2] < GROUND_THR && directionChanged==0) {	// left leaving the line => turn right
							e_set_speed_left((groundValues[0]-GROUND_THR)/2);
							e_set_speed_right(-((groundValues[0]-GROUND_THR)/2));
							directionChanged=1;	
							lineFollowSpeed = 1024-groundValues[0];
						} else if(groundValues[2] > GROUND_THR && groundValues[0] < GROUND_THR && directionChanged==0) {	// right leaving the line => turn left
							e_set_speed_left(-((groundValues[2]-GROUND_THR)/2));
							e_set_speed_right((groundValues[2]-GROUND_THR)/2);
							directionChanged=1;
							lineFollowSpeed = 1024-groundValues[2];
						} else {	// within the line
							if(directionChanged==0) {
								lineFollowSpeed = 300;
								lineFollowSpeed = 300;
							} else {
								if(lineFollowSpeed > 300) {
									lineFollowSpeed=300;
								}
								if(lineFollowSpeed<100) {
									lineFollowSpeed=100;
								}
							}	
							e_set_speed_left(lineFollowSpeed);
							e_set_speed_right(lineFollowSpeed);
							directionChanged=0;
						}

						//wait(10000);
	
						break;

					case 3:	// charge for some time
						//e_ad_scan_off();
						e_set_body_led(0);
						e_set_front_led(0);
						e_led_clear();
						e_destroy_agenda(snake_led);
						e_destroy_agenda(flow_led);
						e_destroy_agenda(k2000_led);
						e_destroy_agenda(right_led);
						e_destroy_agenda(left_led);
	
						while(!e_poxxxx_is_img_ready());	// wait end of capture
	
						if(camImage[0]<170) {	// the contact is lost
							e_poxxxx_launch_capture(&camImage);	// start camera capture
							resetVars();
							//e_ad_scan_on();
							demoState = 2;
							break;
						}

						e_poxxxx_launch_capture(&camImage);	// start camera capture	
	
						if(timerCount > 30) {		// about 30 seconds of charge with camera activated

//							// init LVD
//							_LVDIE = 0;	
//							_LVDIF = 0;
//							//set voltage threshold to 3.1 V
//							RCON=(RCON & 0xF0FF) + 0x0020;
//							_LVDEN=1;
//							while(!_BGST);
//							while(timerCount<32);
//							if(_LVDIF) {	// battery discharged
//								timerCount = 0;
//							} else {
								//e_destroy_agenda(myTimer);
								e_set_steps_left(0);
								e_set_speed_left(-300);
								e_set_speed_right(-300);
								while(e_get_steps_left() > -250);
								//randomDirection = rand()%2;
								randomDirection = 1 - randomDirection;
								if(randomDirection) {
									e_set_steps_left(0);
									e_set_speed_left(300);
									while(e_get_steps_left() < 250);							
								} else {
									e_set_steps_right(0);
									e_set_speed_right(300);
									while(e_get_steps_right() < 250);
								}						
								resetVars();
								timerCount=0;
								//e_ad_scan_on();
								demoState = 0;
//							}
//							_LVDIF=0;
//							_LVDEN=0;

							break;
						}			
						break;
				
			}

		} else if(currentSelector==1) {	// cliff avoidance

			e_i2cp_enable();
    		e_i2cp_read(0xC0, 0);	// in order for the I2C to be initialized well
			e_destroy_agenda(updateGround);
			e_destroy_agenda(myTimer);
			e_set_body_led(0);
			e_set_front_led(0);
			e_led_clear();
			e_set_speed_left(300);
			e_set_speed_right(300);

			while(currentSelector==1) {

				// update ground sensor values
				k = 0;		
				for(l = 0; l < 6; l++) {
					if (l % 2 == 0) { 
						groundTemp[k++] = e_i2cp_read(0xC0, l + 1);
					} else {
						groundTemp[k++] = e_i2cp_read(0xC0, l - 1);
					}
				}
				for(l=13; l<17; l++) {
			    	if (l % 2 == 0) {
						groundTemp[l-7] = e_i2cp_read(0xC0, l - 1);
					} else {
						groundTemp[l-7] = e_i2cp_read(0xC0, l + 1);	          				
					}
				}    

				
				floor_left = (unsigned int) (groundTemp[0] & 0xff) + ((unsigned int) groundTemp[1] << 8);
				floor_right = (unsigned int) (groundTemp[4] & 0xff) + ((unsigned int) groundTemp[5] << 8);
				//floor_center = (((unsigned int) (buffer[2] & 0xff) + ((unsigned int) buffer[3] << 8))+floor_left+floor_right)/3;
				floor_center = (unsigned int) (groundTemp[2] & 0xff) + ((unsigned int) groundTemp[3] << 8);
				cliff_right = (unsigned int) (groundTemp[6] & 0xff) + ((unsigned int) groundTemp[7] << 8);
				cliff_left = (unsigned int) (groundTemp[8] & 0xff) + ((unsigned int) groundTemp[9] << 8);
		
				//delta = floor_right - floor_left;
		
				min_sensor_value = floor_left;
				min_sensor = FLOOR_LEFT;
				if(floor_center < min_sensor_value) {
					min_sensor_value = floor_center;
					min_sensor = FLOOR_CENTER;
				}
				if(floor_right < min_sensor_value) {
					min_sensor_value = floor_right;
					min_sensor = FLOOR_RIGHT;
				}
				if(cliff_right < min_sensor_value) {
					min_sensor_value = cliff_right;
					min_sensor = CLIFF_RIGHT;
				}
				if(cliff_left < min_sensor_value) {
					min_sensor_value = cliff_left;
					min_sensor = CLIFF_LEFT;
				}
		
				// stop if near to falling
				if(min_sensor_value <= FALL_THR) {
		
					e_set_speed_left(0);
					e_set_speed_right(0);
		
					switch(min_sensor) {
						case CLIFF_RIGHT:
							e_set_led(0, 0);
							e_set_led(1, 0);
							e_set_led(2, 1);
							e_set_led(6, 0);
							e_set_led(7, 0);
							break;
						case FLOOR_RIGHT:
							e_set_led(0, 0);
							e_set_led(1, 1);
							e_set_led(2, 0);
							e_set_led(6, 0);
							e_set_led(7, 0);
							break;
						case FLOOR_CENTER:
							e_set_led(0, 1);
							e_set_led(1, 0);
							e_set_led(2, 0);
							e_set_led(6, 0);
							e_set_led(7, 0);
							break;
						case FLOOR_LEFT:
							e_set_led(0, 0);
							e_set_led(1, 0);
							e_set_led(2, 0);
							e_set_led(6, 0);
							e_set_led(7, 1);
							break;
						case CLIFF_LEFT:
							e_set_led(0, 0);
							e_set_led(1, 0);
							e_set_led(2, 0);
							e_set_led(6, 1);
							e_set_led(7, 0);
							break;
					}
		
					e_set_steps_left(0);
					e_set_steps_right(0);
					e_set_speed_left(-200);
					e_set_speed_right(-200);
					while(e_get_steps_left() > -100);
					//for(i=0; i<60000; i++)
					//	for(j=0; j<50; j++);
					e_set_speed_left(0);
					e_set_speed_right(0);
					
					e_set_steps_left(0);
					e_set_steps_right(0);
		
					switch(min_sensor) {
		
						case CLIFF_RIGHT:
						case FLOOR_RIGHT:	// turn left
							e_set_speed_left(-200);
							e_set_speed_right(200);			
							prev_rot = LEFT_ROT;
							while(e_get_steps_left() > -100);		
							//e_set_steps_left(0);
							//e_set_steps_right(0);			
							break;
		
						case FLOOR_CENTER:	// center sensor
							if(prev_rot == LEFT_ROT) {
								e_set_speed_left(-200);
								e_set_speed_right(200);	
								while(e_get_steps_left() > -100);	
								//e_set_steps_left(0);
								//e_set_steps_right(0);			
							} else if(prev_rot == RIGHT_ROT) {
								e_set_speed_left(200);
								e_set_speed_right(-200);	
								while(e_get_steps_left() < 100);
								//e_set_steps_left(0);
								//e_set_steps_right(0);			
							} else {	// turn right by default
								e_set_speed_left(200);
								e_set_speed_right(-200);
								prev_rot = RIGHT_ROT;
								while(e_get_steps_left() < 100);
								//e_set_steps_left(0);
								//e_set_steps_right(0);
							}					
							break;
		
						case FLOOR_LEFT:
						case CLIFF_LEFT:	// turn right
							e_set_speed_left(200);
							e_set_speed_right(-200);
							prev_rot = RIGHT_ROT;
							while(e_get_steps_left() < 100);
							//e_set_steps_left(0);
							//e_set_steps_right(0);					
							break;
		
					}
		
					e_set_speed_left(300);
					e_set_speed_right(300);
		
				} else {
					e_set_led(0, 0);
					e_set_led(1, 0);
					e_set_led(2, 0);
					e_set_led(6, 0);
					e_set_led(7, 0);
				}
		
				for(k=0; k<20000; k++);
				currentSelector = getselector();

			}

			e_i2cp_disable();
			resetVars();
			demoState = 0;
			e_activate_agenda(updateGround, 200);	// 20 ms (50 Hz)
			e_activate_agenda(myTimer, 10000);	// every second

		}  else if(currentSelector==2) {	// test obstacle avoidance at maximum speed desired
			basicSpeed = 1500;
			e_activate_agenda(shock_neuron, 650);

		} else if(currentSelector==3) {	// test obstacle avoidance with randomness

			e_set_body_led(0);
			e_set_front_led(0);
			e_activate_agenda(shock_neuron, 650);

			if((timerCount%5==0) && (lastTimerCount != timerCount)) {	// change state every 10 seconds

				lastTimerCount = timerCount;

				timerCount = 1;

				randomSpeed = rand()%9;
				switch(randomSpeed) {
					case 0: basicSpeed = 550;
							break;
					case 1: basicSpeed = 700;
							break;
					case 2: basicSpeed = 850;
							break;
					case 3: basicSpeed = 1000;
							break;
					case 4: basicSpeed = 1150;
							break;
					case 5: basicSpeed = 1250;
							break;
					case 6: basicSpeed = 1350;
							break;
					case 7: basicSpeed = 1450;
							break;
					case 8: basicSpeed = 1500;
							break;
				}

				randomSound = rand()%5;
				if(currentSelector==1) {
					switch(randomSound) {
						case 0: e_play_sound(0,2112);
								break;
						case 1: e_play_sound(2116,1760);
								break;
						case 2: e_play_sound(3878,3412);
								break;
						case 3: e_play_sound(7294,3732);
								break;
						case 4: e_play_sound(11028,8016);
								break;
					}
				}

				switch(randomLed) {
					case 0: e_destroy_agenda(snake_led);
							break;
					case 1: e_destroy_agenda(flow_led);
							break;
					case 2: e_destroy_agenda(k2000_led);
							break;
					case 3: e_destroy_agenda(right_led);
							break;
					case 4: e_destroy_agenda(left_led);
							break;						
				}
				randomLed = rand()%3;
				switch(randomLed) {
					case 0: e_activate_agenda(snake_led, 1000);
							break;
					case 1: e_activate_agenda(flow_led, 1000);
							break;
					case 2: e_activate_agenda(k2000_led, 1000);
							break;
					case 3: e_activate_agenda(right_led, 1000);
							break;
					case 4: e_activate_agenda(left_led, 1000);
							break;						
				}
				
				randomRotation = rand()%3;
				if(randomRotation==1) {
					e_destroy_agenda(shock_neuron);
					e_set_steps_left(0);
					e_set_speed_left(-700);
					e_set_speed_right(700);
					while(e_get_steps_left() > -500);
					e_activate_agenda(shock_neuron, 650);							
				} else if(randomRotation==2) {
					e_destroy_agenda(shock_neuron);
					e_set_steps_right(0);
					e_set_speed_right(-700);
					e_set_speed_left(700);
					while(e_get_steps_right() > -500);
					e_activate_agenda(shock_neuron, 650);
				}

			}

		} else if (currentSelector==4) { 	// line following and led detecting

			e_poxxxx_launch_capture(&camImage);	// start camera capture


			//e_set_led(2, 1);
			e_set_body_led(1);
			e_set_front_led(0);

			e_destroy_agenda(shock_neuron);

			if(timerCount > 10) {	// the robot seems to be blocked somehow, so after about 30 seconds (with camera activated) restart from state=0
				if(escapeDir == 0) {
					e_set_steps_left(0);
					e_set_speed_left(-150);
					e_set_speed_right(0);
					while(e_get_steps_left() > -75);
					e_set_steps_right(0);
					e_set_speed_left(0);
					e_set_speed_right(-150);
					while(e_get_steps_right() > -75);
				} else {
					e_set_steps_right(0);
					e_set_speed_left(0);
					e_set_speed_right(-150);
					while(e_get_steps_right() > -75);
					e_set_steps_left(0);
					e_set_speed_left(-150);
					e_set_speed_right(0);
					while(e_get_steps_left() > -75);
				}
				timerCount = 0;
				escapeDir = 1 - escapeDir;
			}

			while(!e_poxxxx_is_img_ready());	// wait end of capture

			//sprintf(uartBuff, "%d\n\r", camImage[0]);
			//uart1_send_static_text(uartBuff);

			if(camImage[0]>180) {	//120-10
				//outOfLine = 0;
				chargeContact++;
				if(chargeContact > 10) {
					e_set_speed_left(0);
					e_set_speed_right(0);						
					chargeContact = 10;
					timerCount = 0;
					continue;
				}
			} else {
				chargeContact = 0;
			}

			//if(groundValues[1] < 900) {
			//	e_set_speed_left(300);
			//	e_set_speed_right(300);
			//} else 
//			if(groundValues[0] > 900 && groundValues[2] < 900) {	// left leaving the line => turn right
//				//e_set_speed_left(300);
//				//e_set_speed_right(-300);
//			} else if(groundValues[2] > 900 && groundValues[0] < 900) {	// right leaving the line => turn left
//				e_set_speed_left(-300);
//				e_set_speed_right(300);
//			} else {	// within the line
//				e_set_speed_left(300);
//				e_set_speed_right(300);
//			}

			// Do not let the robot change the direction at every cycle in order to avoid "dancing" when
			// entering in the black line corners.
			// Moreover adapt the forward speed based on how much the robot is outside the line, that is
			// if it is going outside to the right or left then decrease the speed to let the robot react the next
			// cycle.
			if(groundValues[4] < GROUND_THR && groundValues[0]>GROUND_THR && groundValues[1]>GROUND_THR && groundValues[2]>GROUND_THR && groundValues[3]>GROUND_THR) { // left cliff inside line, turn left
				e_set_speed_left(-300);
				e_set_speed_right(300);
				directionChanged=1;
			} else if(groundValues[3] < GROUND_THR && groundValues[0]>GROUND_THR && groundValues[1]>GROUND_THR && groundValues[2]>GROUND_THR && groundValues[4]>GROUND_THR) {
				e_set_speed_left(300);
				e_set_speed_right(-300);
				directionChanged=1;
			} else if(groundValues[0] > GROUND_THR && groundValues[2] < GROUND_THR && directionChanged==0) {	// left leaving the line => turn right
				e_set_speed_left((groundValues[0]-GROUND_THR)/2);
				e_set_speed_right(-((groundValues[0]-GROUND_THR)/2));
				directionChanged=1;	
				lineFollowSpeed = 1024-groundValues[0];
			} else if(groundValues[2] > GROUND_THR && groundValues[0] < GROUND_THR && directionChanged==0) {	// right leaving the line => turn left
				e_set_speed_left(-((groundValues[2]-GROUND_THR)/2));
				e_set_speed_right((groundValues[2]-GROUND_THR)/2);
				directionChanged=1;
				lineFollowSpeed = 1024-groundValues[2];
			} else {	// within the line
				if(directionChanged==0) {
					lineFollowSpeed = 300;
					lineFollowSpeed = 300;
				} else {
					if(lineFollowSpeed > 300) {
						lineFollowSpeed=300;
					}
					if(lineFollowSpeed<100) {
						lineFollowSpeed=100;
					}
				}	
				e_set_speed_left(lineFollowSpeed);
				e_set_speed_right(lineFollowSpeed);
				directionChanged=0;
			}

			//wait(10000);

		} else if(currentSelector==5) {
			e_poxxxx_launch_capture(&camImage);	// start camera capture
			while(!e_poxxxx_is_img_ready());	// wait end of capture
			sprintf(uartBuff, "%d, %d, %d, %d, %d, %d\n\r", groundValues[0], groundValues[1], groundValues[2], groundValues[3], groundValues[4], camImage[0]);
			uart1_send_static_text(uartBuff);
		} else {	// do nothing
			e_set_speed_left(0);
			e_set_speed_right(0);
			e_led_clear();
			lineFound = 0;
			timerCount = 0;
			outOfLine = 0;
			chargeContact = 0;
			demoState = 0;
		}

	}

	return 0;
}

